#!/bin/sh

VERSION=1.2.0

BOARD=$1
UBOOT=$2
KERNEL=$3
DTB=$4
USERLAND=$5
SD_KERNEL=$6
SD_DTB=$7
USER_PART_SIZE=$8

MAX_USER_PART_SIZE=1400
MIN_USER_PART_SIZE=1
BOARD_LIST="a640 a610 aiota6"

check_params ()
{
	if [ -z "$SD_KERNEL" ]; then
		SD_KERNEL="$KERNEL"
	fi
	if [ -z "$SD_DTB" ]; then
		SD_DTB="$DTB"
	fi

	if [ -n "$USER_PART_SIZE" ]; then
		if ! [ "$USER_PART_SIZE" -eq "$USER_PART_SIZE" ] 2> /dev/null; then
			echo "ERROR: USER_PART_SIZE must be numeric"
			exit 1
		fi

		if [ "$USER_PART_SIZE" -gt "$MAX_USER_PART_SIZE" ] ||
			 [ "$USER_PART_SIZE" -lt "$MIN_USER_PART_SIZE" ]; then
			echo "ERROR: USER_PART_SIZE must be $MIN_USER_PART_SIZE - $MAX_USER_PART_SIZE(MB)"
			exit 1
		fi
	fi

	IMAGES="$UBOOT $KERNEL $DTB $USERLAND $SD_KERNEL $SD_DTB"
	for FILE in $IMAGES; do
		if [ ! -r $FILE ]; then
			echo "ERROR: $FILE: No such file or directory"
			exit 1
		fi
	done
	case "$BOARD" in
		"a640")
			DTB_NAME="a640.dtb"
			SD_UBOOT="core/u-boot.imx"
			;;
		"a610")
			DTB_NAME="a610.dtb"
			SD_UBOOT="core/u-boot.imx"
			;;
		"aiota6")
			DTB_NAME="a640.dtb"
			SD_UBOOT="core/u-boot-uart3.imx"
			if [ -z "$USER_PART_SIZE" ]; then
				USER_PART_SIZE="1024"
			fi
			;;
		*)
			echo "ERROR: Unkown $BOARD."
			usage
			exit 1
			;;
	esac
}

get_md5 ()
{
	md5sum $1 | awk '{ print $1 }'
}

create_install_conf ()
{
	echo "#!/bin/sh
#
# install-image configuration
#
LOGFILE=/root/install.log
SEND_LOGFILE=/root/send_log.log

#
# Files
#
BOOTLOADER_IMAGE=/images/u-boot.imx
BOOTLOADER_IMAGE_MD5=$(get_md5 $UBOOT)

KERNEL_IMAGE=/images/uImage
KERNEL_IMAGE_MD5=$(get_md5 $KERNEL)

DEVICETREE_IMAGE=/images/$DTB_NAME
DEVICETREE_IMAGE_MD5=$(get_md5 $DTB)

ROOTFS_ARCHIVE=/images/$(basename $USERLAND)
ROOTFS_ARCHIVE_MD5=$(get_md5 $USERLAND)"

	if [ -n "$USER_PART_SIZE" ]; then
		echo "USER_PART_SIZE=+${USER_PART_SIZE}M"
	fi
}

clean ()
{
	if mountpoint -q $MNTDIR ; then
		umount $MNTDIR
	fi
	rm -rf $MNTDIR
}

init_images ()
{
	TEMP=$(mktemp -d)
	mkdir $TEMP/p1 $TEMP/p2

	# Init p1
	# nothing to do

	# Init p2
	tar xf core/dist.tar.gz -C $TEMP/p2
	mkdir -p $TEMP/p2/boot
	cp $SD_KERNEL $TEMP/p2/boot/uImage
	cp $SD_DTB $TEMP/p2/boot/$DTB_NAME
	cp core/images/* $TEMP/p2/images/

	create_install_conf > $TEMP/p2/images/install.conf

	cp $UBOOT $TEMP/p2/images/u-boot.imx
	cp $USERLAND $TEMP/p2/images/
	cp $KERNEL $TEMP/p2/images/uImage
	cp $DTB $TEMP/p2/images/$DTB_NAME
}

calc_disk_size_mb ()
{
	# Partition 1: total size of files + 1MB margin
	SIZE_DISK_P1=$(du -sm $TEMP/p1 | awk '{print $1}')
	SIZE_DISK_P1=$(( SIZE_DISK_P1 + 1 ))

	# Partition 2: total size of files + 50MB margin
	SIZE_DISK_P2=$(du -sm $TEMP/p2 | awk '{print $1}')
	SIZE_DISK_P2=$(( SIZE_DISK_P2 + 50 ))

	SIZE_DISK=$(( SIZE_DISK_P1 + SIZE_DISK_P2 ))
}

create_disk ()
{
	INSTALL_IMAGE="install-disk-sd-${BOARD}-$(date +%Y%m%d).img"
	rm -f ${INSTALL_IMAGE}

	# Make empty install image
	dd of=${INSTALL_IMAGE} count=0 seek=1 bs=${SIZE_DISK}M

	# Attach install image to loop device
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Edit partitions
	sfdisk ${DISK_DEV_TMP} <<EOF
,${SIZE_DISK_P1}M,b
,${SIZE_DISK_P2}M,83
EOF

	# Re-attach install image to loop device for partitions
	losetup -d ${DISK_DEV_TMP}
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Format partitions
	mkfs.vfat ${DISK_DEV_TMP}p1
	mkfs.ext4 -F ${DISK_DEV_TMP}p2

	MNTDIR=$(mktemp -d)

	# Copy files to partition 1
	# nothing to do

	# Copy files to partition 2
	mount ${DISK_DEV_TMP}p2 $MNTDIR
	cp -r $TEMP/p2/* $MNTDIR
	sync
	umount $MNTDIR

	rmdir $MNTDIR

	# Write u-boot image
	dd if=${SD_UBOOT} of=${DISK_DEV_TMP} bs=1k seek=1 conv=fsync
	sync

	# Detach loop device
	losetup -d ${DISK_DEV_TMP}
}

usage ()
{
	echo "Install Disk Image Build Script v$VERSION

Usage:
  sudo ./build.sh BOARD UBOOT KERNEL DTB USERLAND [SD_KERNEL SD_DTB USER_PART_SIZE]

  BOARD: $(echo $BOARD_LIST | sed 's/ /\//g')
  UBOOT: u-boot image
  KERNEL: uImage
  DTB: Device Tree Blob image
  USERLAND: Debian userland archive
  SD_KERNEL: uImage for install disk(optional)
  SD_DTB: Device Tree Blob image for install disk(optional)
  USER_PART_SIZE: eMMC(/dev/mmcblk0p4) user partition size(optional) $MIN_USER_PART_SIZE - $MAX_USER_PART_SIZE (MB)

Example:
  incase of using default images as:

    sudo ./build.sh a640 u-boot-a600-v2018.03-at4.imx uImage-a600-v4.14-at11 armadillo-640-v4.14-at11.dtb debian-stretch-armhf-a600-20190326.tar.gz

  incase of using user-customized images as(that images use uSDHC2 via Armadillo-610 CON2 Ext-Interface):

    sudo ./build.sh a610 u-boot.imx uImage a610.dtb debian-stretch-armhf-a600-20200214.tar.gz core/uImage core/a610.dtb"
}

if [ $(whoami) != "root" ] ;then
	echo "ERROR: Please run script with sudo"
	exit 1
fi

if [ $# -lt 5 ]; then
	usage
	exit 1
fi

trap 'clean ; exit 1' 1 2 3 15

check_params
init_images
calc_disk_size_mb
create_disk
clean
